# Supports Organizer.rb
=begin rdoc
= Camera Recorder
Copyright 2009, Chris Fullmer
All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.

== License
This software is distributed under the Smustard End User License Agreement
http://www.smustard.com/eula

== Information
Author:: Chris Fullmer
Organization:: www.ChrisFullmer.com and distributed on sketchucation.com
Name:: Camera Recorder
Version:: 1.5
SU Version:: I developed this on v7, but old versions should work.
Date:: November 11th, 2009
Description:: This script records the navigating of a model and exports it out as a series of images to be turned into a movie using quicktime or any other capable video editor.
Usage::  It is installed in Plugins > Chris Fullmer Tools > Camera Recorder > (items).  Start recording and navigate the model.  Then stop recording.  Clear the recording if you desire.  Or export the animation into an existing folder.  Supports jpg and png file types.
Requires:: No additional Scripts
History::
1.0:: 2009-11-10
   * Original release.  
1.5:: 2009-11-11
   * Fixed the lag bug that occured in the beginning of each recording
	* Added more user feedback when buttons are pressed in the wrong order
	* Added the abilty to control image output height, width and anti-aliasing
	* Added better file names so that more software will correctly order the image frames
	* Added a recording playback button, so that recordings can be played back before exporting.
	* Added a tool to place constructions points at the location of each frame, with another construction point where the camera is pointing.
	* Added some new icons, but they still need help.
1.7:: 2009-11-17
	* Added the ability to export frames to scenes
	* Script now remembers user settings from SketchUp session to session
=end

class Clf_ViewObs < Sketchup::ViewObserver
  
	@@eye = [] 
	@@direction = [] 
	@@up = [] 
	@@recording = false
	@@pts_exist = false
	@@path = ""
	@@pc = Object::RUBY_PLATFORM.include?("mswin")
	@@a = Sketchup.version
	@@b = @@a[0,1].to_i
	if @@b > 6
		@@fast = true
	else
		@@fast = false
	end
	
	#---------Read Defaults, the 3rd value is only used if there is no default already present----------
	@@fps = Sketchup.read_default( 'clf_camera_recorder', 'fps', '15' )
	@@export_type = Sketchup.read_default( 'clf_camera_recorder', 'export_type', 'Scenes' )
	@@base_file_name = Sketchup.read_default( 'clf_camera_recorder', 'base_file_name', 'image' )
	@@file_type = Sketchup.read_default( 'clf_camera_recorder', 'file_type', 'png' )
	@@image_width = Sketchup.read_default( 'clf_camera_recorder', 'image_width', '800' )
	@@image_height = Sketchup.read_default( 'clf_camera_recorder', 'image_height', '600' )
	@@image_anti_alias = Sketchup.read_default( 'clf_camera_recorder', 'image_anti_alias', 'Yes' )
	@@first_run = Sketchup.read_default( 'clf_camera_recorder', 'first_run', 'yes' )

	#----------Write defaults, will re-write any existing defaults------------
	if @@first_run == 'yes'
		Sketchup.write_default( 'clf_camera_recorder', 'first_run', 'no' )
		Sketchup.write_default( 'clf_camera_recorder', 'fps', @@fps )
		Sketchup.write_default( 'clf_camera_recorder', 'export_type', @@export_type )

		Sketchup.write_default( 'clf_camera_recorder', 'base_file_name', @@base_file_name )
		Sketchup.write_default( 'clf_camera_recorder', 'file_type', @@file_type )
		Sketchup.write_default( 'clf_camera_recorder', 'image_width', @@image_width )
		Sketchup.write_default( 'clf_camera_recorder', 'image_height', @@image_height )
		Sketchup.write_default( 'clf_camera_recorder', 'image_anti_alias', @@image_anti_alias )
	end
	
	def export_images
		prompt = ["Destination Folder", "Base File Name", "File Type", "Image Width", "Image Height", "Anti-Alias?"]
		value = ["", @@base_file_name, @@file_type, @@image_width.to_i, @@image_height.to_i, @@image_anti_alias]
		list = ["","","png|jpg", "", "", "Yes|No"]
		results = UI.inputbox( prompt, value, list, "Export Options" )
		if results
			@@destination_folder = results[0]
			@@base_file_name = results[1]
			full_file_name = @@base_file_name+"00000"
			@@file_type = results[2]
			@@image_width = results[3]
			@@image_height = results[4]
			@@image_anti_alias = results[5]
			if @@pc
				Sketchup.write_default( 'clf_camera_recorder', 'destination_folder', @@destination_folder.tr('\\', '/' ) )
			else
				Sketchup.write_default( 'clf_camera_recorder', 'destination_folder', @@destination_folder.gsub( /\\/, "/" ) )
			end
			Sketchup.write_default( 'clf_camera_recorder', 'base_file_name', @@base_file_name.to_s )
			Sketchup.write_default( 'clf_camera_recorder', 'file_type', @@file_type )
			Sketchup.write_default( 'clf_camera_recorder', 'image_width', @@image_width )
			Sketchup.write_default( 'clf_camera_recorder', 'image_height', @@image_height )
			Sketchup.write_default( 'clf_camera_recorder', 'image_anti_alias', @@image_anti_alias )
			if @@image_anti_alias == "Yes"
				image_aa = true
			else
				image_aa = false
			end
			puts "Exporting"
			view = Sketchup.active_model.active_view
			
			@@eye.each_index do |e|
				Sketchup.status_text = "Exporting #{e} of #{@@eye.length} images."
				cam = cam_maker( e )
				view.camera=cam
				img_full_name = File.join( @@destination_folder, full_file_name+"."+@@file_type )
				view.write_image( img_full_name, @@image_width, @@image_height, image_aa )
				full_file_name=full_file_name.next
			end
			Sketchup.status_text = "Done Exporting!"
			puts "Done!"
		else
			puts "Cancelled"
		end
	end
	
	
	def record
		if !@@recording
			if @@eye.length > 0
				cancel = clear_rec
				if cancel == "cancel"
					UI.messagebox "Recording Cancelled.  You must clear previous recording first."
					return
				end
			end
			prompts = ["How many FPS?"]
			defaults = [@@fps]
			results = UI.inputbox( prompts, defaults, "Camera Recorder" )
			if results
				@@recording = true
				@@fps = results[0]
				Sketchup.write_default( 'clf_camera_recorder', 'fps', @@fps.to_s )
				@@speed = 1.0/@@fps.to_f
				@@frame = 0
				@@time = Time.now
			else
				puts "Cancelled"
				Sketchup.active_model.active_view.remove_observer($clf_cam_ob)
			end
		else
			Sketchup.status_text = "Already recording!"
		end
	end
	
	
	def stop_rec
		Sketchup.active_model.active_view.remove_observer($clf_cam_ob)
		if @@eye.length > 0
			UI.messagebox @@eye.length.to_s + " frames recorded."
			@@recording = false
			Sketchup.status_text = "Recording Stopped!"
		else
			UI.messagebox "Nothing has been recorded.  Please press Record first."
		end
	end # method stop_rec
	
	# This clears the recording.  It first checks if there is a recording in memmory.
	# It then pops up a Yes or No messagebox if there is a recording, or alerts the user that
	# nothing had been recorded.  If they choose to clear the recording, it clears it.  Simple
	def clear_rec
		if @@eye.length > 0
			continue = UI.messagebox "Clear recording?", MB_YESNO
			if continue == 6
				@@time = []
				@@direction = []
				@@up = []
				@@eye = []
				@@pts_exist = false
				puts "cleared"
			else
				puts "clear cancelled"
				return "cancel"
			end
		else
			UI.messagebox "No recording to clear!"
		end
	end # Method
	
	
	def draw_curve
		Sketchup.active_model.start_operation "Add Curve", @@fast
		if @@eye.length > 0
			if @@pts_exist
				results = UI.messagebox "Points have already been created for this Animation, re-output them?", MB_YESNO
				if results == 6
					continue = true
				else
					continue = false
				end
			else
				continue = true
			end
		
			if continue
				model = Sketchup.active_model
				ents = model.entities
				g = ents.add_group
				gents = g.entities
				@@eye.each_index do |e| 
					gents.add_cpoint( @@eye[e] )
					@@direction[e].length = 100
					target = @@eye[e].clone.offset @@direction[e]
					gents.add_cpoint( target )
				end
				g.locked=true
				@@pts_exist = true
			else
				puts "recording existed, but re-export was cancelled."
			end
		else
			UI.messagebox "No recording to create points for."
		end
		Sketchup.active_model.commit_operation
	end
	
	
	def cam_maker( index )
		Sketchup::Camera.new( @@eye[index], @@direction[index], @@up[index] )
	end
	
	def make_scenes
		Sketchup.active_model.start_operation "Make Scenes from Frames", @@fast
		pages = Sketchup.active_model.pages
		view = Sketchup.active_model.active_view
		pg_name = "f"
		pg_num = 1
		@@eye.each_index do |index|
			cam = cam_maker( index )
			view.camera=cam
			page = pages.add pg_name+pg_num.to_s, 1
			pg_num = pg_num + 1
		end
		Sketchup.active_model.commit_operation
	end
	
	
	def animate
		if @@eye.length >0
			Sketchup.active_model.active_view.animation = self
		else
			UI.messagebox "There is no recording to replay."
		end
	end
	
	
	def nextFrame(view)
		cam = cam_maker( @@frame )
		view.camera = cam
		view.show_frame @@speed
		if @@frame+1 < @@eye.length
			@@frame = @@frame+1
			return true
		else
			@@frame = 0
			return false
		end
	end

	
	def export
		if @@eye.length > 0
			prompt = ["Export as: "]
			defaults = [@@export_type]
			values = ["Scenes|Images"]
			results = UI.inputbox( prompt, defaults, values, "Export Type" )
			@@export_type = results[0]
			Sketchup.write_default( 'clf_camera_recorder', 'export_type', @@export_type )
			make_scenes if @@export_type == "Scenes"
			export_images if @@export_type == "Images"
		else
			UI.messagebox "There is no recording to export."
		end
	end
	

	
	def onViewChanged(view)
			elapsed = Time.now - @@time
			missed_views = (elapsed/@@speed).to_i
			if elapsed >= @@speed
				missed_views.times do
					@@eye << view.camera.eye
					@@direction << view.camera.direction
					@@up << view.camera.up
					@@time = Time.now
				end
			end
	end # method

end # Class

	  
# ---------------- MENU-------------------

if( $submenu )
    organizer = $submenu 
	if !file_loaded?(__FILE__) then
		clf_cam_recorder = organizer.add_submenu("Camera Recorder")
		$clf_cam_ob = Clf_ViewObs.new
		toolbar = UI::Toolbar.new "Camera Recorder"
		cmd1 = UI::Command.new("Start Recording") do 
			Sketchup.active_model.active_view.add_observer($clf_cam_ob)
			$clf_cam_ob.record
		end
		cmd2 = UI::Command.new("Stop Recording") { $clf_cam_ob.stop_rec }
		cmd3 = UI::Command.new("Erase Recording") { $clf_cam_ob.clear_rec }
		cmd4 = UI::Command.new("Export Recording") { $clf_cam_ob.export }
    
		clf_cam_recorder.add_item cmd1
		clf_cam_recorder.add_item cmd2
		clf_cam_recorder.add_item cmd3
		clf_cam_recorder.add_item cmd4
		
		cmd1.small_icon = "green_16.png"
		cmd1.large_icon = "green_24.png"
		cmd2.small_icon = "red_16.png"
		cmd2.large_icon = "red_24.png"
		cmd3.small_icon = "x_16.png"
		cmd3.large_icon = "x_24.png"
		cmd4.small_icon = "movie_16.png"
		cmd4.large_icon = "movie_24.png"
		
		cmd1.tooltip = "Start Recording"
		cmd2.tooltip = "Stop Recording"
		cmd3.tooltip = "Erase Recording"
		cmd4.tooltip = "Export Recording"
		
		toolbar = toolbar.add_item cmd1
		toolbar = toolbar.add_item cmd2
		toolbar = toolbar.add_item cmd3
		toolbar = toolbar.add_item cmd4
		
		cmd5 = UI::Command.new("Play Animation") { $clf_cam_ob.animate }
		clf_cam_recorder.add_item cmd5
		cmd5.small_icon = "player_play-16.png"
		cmd5.large_icon = "player_play-24.png"
		cmd5.tooltip = "Play Animation"
		toolbar = toolbar.add_item cmd5
		
		cmd6 = UI::Command.new("Create Curve") { $clf_cam_ob.draw_curve }
		clf_cam_recorder.add_item cmd6
		cmd6.small_icon = "1_16.png"
		cmd6.large_icon = "1_24.png"
		cmd6.tooltip = "Create Curve"
		toolbar = toolbar.add_item cmd6
		
		
		case toolbar.get_last_state
		when 1
			toolbar.restore
		when -1   
			toolbar.show
		end
	end
else
	if !file_loaded?('clf_menu_loader') then
        $clf_tools_menu = UI.menu("Plugins").add_submenu("Chris Fullmer Tools")
   end
   	#------New menu Items---------------------------
	if !file_loaded?('clf_youtube_loader')
		$clf_youtube_menu = $clf_tools_menu.add_submenu("-=YouTube Video Tutorials For My Scripts=-")
		$clf_youtube_menu.add_item("My YouTube Channel"){UI.openURL "http://www.youtube.com/user/ChrisFullmer"}
	end
	if !file_loaded?('clf_myscipts_loader')
		$clf_tools_menu.add_item("-=All My Plugins I've Written=-"){UI.openURL "http://forums.sketchucation.com/search.php?keywords=plugin&terms=all&author=Chris+Fullmer&sc=1&sf=titleonly&sr=topics&sk=t&sd=d&st=0&ch=300&t=0&submit=Search"}
	end
	#------YouTube for this Script------------------
	$clf_youtube_menu.add_item("Camera Recorder"){UI.openURL "http://www.chrisfullmer.com/plugins/clf_camera_recorder.html"}		
	#------Donate to the Author--------------------
	if !file_loaded?('clf_paypal_loader')
		$clf_tools_menu.add_item("-=Donate $$$ to Chris via PayPal=-"){UI.openURL "http://www.chrisfullmer.com/plugins/donate.html"}
		$clf_tools_menu.add_separator
	end
	#------------------------------------------------
	if !file_loaded?(__FILE__) then
		$clf_cam_ob = Clf_ViewObs.new
		clf_cam_recorder = $clf_tools_menu.add_submenu("Camera Recorder")
		
		toolbar = UI::Toolbar.new "Camera Recorder"
		
		cmd1 = UI::Command.new("Start Recording") do 
			Sketchup.active_model.active_view.add_observer($clf_cam_ob)
			$clf_cam_ob.record
		end
		cmd2 = UI::Command.new("Stop Recording") { $clf_cam_ob.stop_rec }
		cmd3 = UI::Command.new("Erase Recording") { $clf_cam_ob.clear_rec }
		cmd4 = UI::Command.new("Export Recording") { $clf_cam_ob.export }
    
		clf_cam_recorder.add_item cmd1
		clf_cam_recorder.add_item cmd2
		clf_cam_recorder.add_item cmd3
		clf_cam_recorder.add_item cmd4
		
		cmd1.small_icon = "green_16.png"
		cmd1.large_icon = "green_24.png"
		cmd2.small_icon = "red_16.png"
		cmd2.large_icon = "red_24.png"
		cmd3.small_icon = "x_16.png"
		cmd3.large_icon = "x_24.png"
		cmd4.small_icon = "movie_16.png"
		cmd4.large_icon = "movie_24.png"
		
		cmd1.tooltip = "Start Recording"
		cmd2.tooltip = "Stop Recording"
		cmd3.tooltip = "Erase Recording"
		cmd4.tooltip = "Export Recording"
		
		toolbar = toolbar.add_item cmd1
		toolbar = toolbar.add_item cmd2
		toolbar = toolbar.add_item cmd3
		toolbar = toolbar.add_item cmd4
		
		cmd5 = UI::Command.new("Play Animation") { Sketchup.active_model.active_view.animation = $clf_cam_ob.animate }
		clf_cam_recorder.add_item cmd5
		cmd5.small_icon = "player_play-16.png"
		cmd5.large_icon = "player_play-24.png"
		cmd5.tooltip = "Play Animation"
		toolbar = toolbar.add_item cmd5
		
		cmd6 = UI::Command.new("Create Curve") { $clf_cam_ob.draw_curve }
		clf_cam_recorder.add_item cmd6
		cmd6.small_icon = "1_16.png"
		cmd6.large_icon = "1_24.png"
		cmd6.tooltip = "Create Curve"
		toolbar = toolbar.add_item cmd6
		
		case toolbar.get_last_state
		when 1
			toolbar.restore
		when -1   
			toolbar.show
		end
	end
end

file_loaded('clf_myscipts_loader')
file_loaded('clf_paypal_loader')
file_loaded('clf_youtube_loader')
file_loaded('clf_menu_loader')
file_loaded('clf_menu_loader')
file_loaded(__FILE__)